/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2007 University of Washington
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 */

// Author: Baltzis Petros<mpaltzis@ceid.upatras.gr>

#ifndef LEAKYBUCKET_H
#define LEAKYBUCKET_H

#include <queue>
#include "ns3/packet.h"
#include "ns3/queue.h"
#include "ns3/data-rate.h"
#include "ns3/address.h"
#include "ns3/ptr.h"
#include "ns3/ipv4-header.h"


namespace ns3 {

class TraceContainer;


/**
 * To leaky bucket ulopoiei einai upoklash ths Queue.Etsi ulopoiei enan 
 * mhxanismo fifo aporiptontas ola ta paketa pou erxontai efoson gemisei to bucket. 
 * 
 */
class LeakyBucket : public Queue {
public:
  static TypeId GetTypeId (void);
  /**
   * LeakyBucket Constructor
   *
   * Dhmiourgei ena LeakyBucket me oura megethous 100 paketwn by default
   */
  LeakyBucket ();

  virtual ~LeakyBucket();

  /**
   * Katametrish twn tropwn metrisis megethous ths ouras 
   *
   */
  enum Mode {
    ILLEGAL,     
    PACKETS,     
    BYTES,        
  };


  /**
   * Orismos tou mode ths sugkekrimenhs syskeuhs
   *
   * To by default mode gia to megethos ths ouras einai ta packets
   *
   */
  void SetMode (LeakyBucket::Mode mode);

  /**
   * Epistrefei to mode ths sugkekrimenhs suskeuhs
   * 
   */
  LeakyBucket::Mode  GetMode (void);

  /**
   * Epistrefei thn ip tou kombou pou tha diamorfwsei thn kinhsh
   * 
   */
  Ipv4Address GetIpv4Shape(void);

  /**
   * Epistrefei to cbr gia thn apostolh twn paketwn apo to bucket
   * 
   */
  DataRate GetCbrRate(void);

private:
  //Sunarthsh pou epistreifei true ean egine enqueue to packet
  virtual bool DoEnqueue (Ptr<Packet> p);
  //Sunarthsh pou kanei dequeue to packet pou brisketai mprosta sthn oura
  virtual Ptr<Packet> DoDequeue (void);
  //Sunarthsh pou epistrefei to paketo pou brisketai mprosta sthn oura
  virtual Ptr<const Packet> DoPeek (void) const;

  std::queue<Ptr<Packet> > m_packets;
  uint32_t      m_maxPackets;
  uint32_t      m_maxBytes;
  uint32_t      m_bytesInQueue;       
  Mode          m_mode;               // Tropos megethou ths ouras
  DataRate      m_cbrRate;            // Data rate gia to leaky bucket
  uint8_t       m_DSCP;               // H DSCP timh
  Ipv4Address   m_peer;               // Ip dieuthinsi tou kombou gia morfopoihsh
};

}; // namespace ns3

#endif /* DROPTAIL_H */
